/*
 * $Id$
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Ecore.h>
#include <stdbool.h>

#include "rb_ecore.h"

typedef struct {
	Ecore_Idler *idler;
	void *cb;
	bool deleted;
} RbEcoreIdler;

VALUE cIdler;

static int on_idler (void *data)
{
	VALUE r;
	RbEcoreIdler *idler = data;

	r = rb_funcall ((VALUE) idler->cb, rb_intern ("call"), 0);

	/* if the callback returns false, we return 0 and Ecore
	 * will remove the idler
	 */
	if (r == Qfalse)
		idler->deleted = true;

	return (r != Qfalse);
}

static VALUE c_init (VALUE self)
{
	RbEcoreIdler *idler = NULL;

	Data_Get_Struct (self, RbEcoreIdler, idler);

	idler->cb = (void *) rb_block_proc ();
	idler->idler = ecore_idler_add (on_idler, idler);

	return self;
}

static void c_free (RbEcoreIdler *idler)
{
	if (idler->idler && !idler->deleted)
		ecore_idler_del (idler->idler);

	free (idler);
}

static VALUE c_new (VALUE klass)
{
	VALUE self;
	RbEcoreIdler *idler;

	self = Data_Make_Struct (klass, RbEcoreIdler, NULL, c_free, idler);

	rb_obj_call_init (self, 0, NULL);

	return self;
}

static VALUE c_delete (VALUE self)
{
	RbEcoreIdler *idler = NULL;

	Data_Get_Struct (self, RbEcoreIdler, idler);

	if (idler->idler && !idler->deleted) {
		ecore_idler_del (idler->idler);
		idler->idler = NULL;
		idler->deleted = true;
	} else
		rb_raise (rb_eException, "Idler already deleted!");

	return Qnil;
}

void Init_Idler (void)
{
	cIdler = rb_define_class_under (mEcore, "Idler", rb_cObject);

	rb_define_singleton_method (cIdler, "new", c_new, 1);
	rb_define_method (cIdler, "initialize", c_init, 1);
	rb_define_method (cIdler, "delete", c_delete, 0);
}

