/*
 * $Id$
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Ecore_Evas.h>

#include "rb_ecore_evas_main.h"
#include "rb_ecore_evas.h"

VALUE cGlX11;

static VALUE c_init (int argc, VALUE *argv, VALUE self)
{
	VALUE disp, parent, geom[4];
	Ecore_Evas **ee = NULL;
	char *cdisp = NULL;
	int i, igeom[4] = {0};

	Data_Get_Struct (self, Ecore_Evas *, ee);

	rb_scan_args (argc, argv, "06", &disp, &parent,
	              &geom[0], &geom[1], &geom[2], &geom[3]);

	if (!NIL_P (disp)) {
		Check_Type (disp, T_STRING);
		cdisp = STR2CSTR (disp);
	}

	for (i = 0; i < 4; i++)
		if (!NIL_P (geom[i])) {
			Check_Type (geom[i], T_FIXNUM);
			igeom[i] = NUM2INT (geom[i]);
		}

	*ee = ecore_evas_gl_x11_new (cdisp, 0,
	                             igeom[0], igeom[1],
	                             igeom[2], igeom[3]);

	return self;
}

static VALUE c_new (int argc, VALUE *argv, VALUE klass)
{
	VALUE self;
	Ecore_Evas **ee;

	self = Data_Make_Struct (klass, Ecore_Evas *,
	                         NULL, c_ecore_evas_free, ee);

	rb_obj_call_init (self, argc, argv);

	return self;
}

void Init_GlX11 (void)
{
	cGlX11 = rb_define_class_under (mEvas, "GlX11", cEcoreEvas);

	rb_define_singleton_method (cGlX11, "new", c_new, -1);
	rb_define_method (cGlX11, "initialize", c_init, -1);
}

